@@ -25,11 +25,14 @@
     implementation is provided for convenience and demonstrational
     purposes only.
 """
+from __future__ import annotations
 
+from typing import IO
 
 from . import ImageFile, ImagePalette, UnidentifiedImageError
 from ._binary import i16be as i16
 from ._binary import i32be as i32
+from ._typing import StrOrBytesPath
 
 
 class GdImageFile(ImageFile.ImageFile):
@@ -43,16 +46,17 @@ class GdImageFile(ImageFile.ImageFile):
     format = "GD"
     format_description = "GD uncompressed images"
 
-    def _open(self):
-
+    def _open(self) -> None:
         # Header
+        assert self.fp is not None
+
         s = self.fp.read(1037)
 
-        if not i16(s) in [65534, 65535]:
+        if i16(s) not in [65534, 65535]:
             msg = "Not a valid GD 2.x .gd file"
             raise SyntaxError(msg)
 
-        self.mode = "L"  # FIXME: "P"
+        self._mode = "P"
         self._size = i16(s, 2), i16(s, 4)
 
         true_color = s[6]
@@ -64,20 +68,20 @@ class GdImageFile(ImageFile.ImageFile):
             self.info["transparency"] = tindex
 
         self.palette = ImagePalette.raw(
-            "XBGR", s[7 + true_color_offset + 4 : 7 + true_color_offset + 4 + 256 * 4]
+            "RGBX", s[7 + true_color_offset + 6 : 7 + true_color_offset + 6 + 256 * 4]
         )
 
         self.tile = [
-            (
+            ImageFile._Tile(
                 "raw",
                 (0, 0) + self.size,
-                7 + true_color_offset + 4 + 256 * 4,
-                ("L", 0, 1),
+                7 + true_color_offset + 6 + 256 * 4,
+                "L",
             )
         ]
 
 
-def open(fp, mode="r"):
+def open(fp: StrOrBytesPath | IO[bytes], mode: str = "r") -> GdImageFile:
     """
     Load texture from a GD image file.
 
