@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from io import BytesIO
 
 from . import Image, ImageFile
@@ -9,10 +11,9 @@ try:
 except ImportError:
     SUPPORTED = False
 
-
-_VALID_WEBP_MODES = {"RGBX": True, "RGBA": True, "RGB": True}
-
-_VALID_WEBP_LEGACY_MODES = {"RGB": True, "RGBA": True}
+TYPE_CHECKING = False
+if TYPE_CHECKING:
+    from typing import IO, Any
 
 _VP8_MODES_BY_IDENTIFIER = {
     b"VP8 ": "RGB",
@@ -21,8 +22,8 @@ _VP8_MODES_BY_IDENTIFIER = {
 }
 
 
-def _accept(prefix):
-    is_riff_file_format = prefix[:4] == b"RIFF"
+def _accept(prefix: bytes) -> bool | str:
+    is_riff_file_format = prefix.startswith(b"RIFF")
     is_webp_file = prefix[8:12] == b"WEBP"
     is_valid_vp8_mode = prefix[12:16] in _VP8_MODES_BY_IDENTIFIER
 
@@ -32,98 +33,66 @@ def _accept(prefix):
                 "image file could not be identified because WEBP support not installed"
             )
         return True
+    return False
 
 
 class WebPImageFile(ImageFile.ImageFile):
-
     format = "WEBP"
     format_description = "WebP image"
+    __loaded = 0
+    __logical_frame = 0
 
-    def _open(self):
-        if not _webp.HAVE_WEBPANIM:
-            # Legacy mode
-            data, width, height, self.mode, icc_profile, exif = _webp.WebPDecode(
-                self.fp.read()
-            )
-            if icc_profile:
-                self.info["icc_profile"] = icc_profile
-            if exif:
-                self.info["exif"] = exif
-            self._size = width, height
-            self.fp = BytesIO(data)
-            self.tile = [("raw", (0, 0) + self.size, 0, self.mode)]
-            self._n_frames = 1
-            return
-
+    def _open(self) -> None:
         # Use the newer AnimDecoder API to parse the (possibly) animated file,
         # and access muxed chunks like ICC/EXIF/XMP.
+        assert self.fp is not None
         self._decoder = _webp.WebPAnimDecoder(self.fp.read())
 
         # Get info from decoder
-        width, height, loop_count, bgcolor, frame_count, mode = self._decoder.get_info()
-        self._size = width, height
-        self.info["loop"] = loop_count
-        bg_a, bg_r, bg_g, bg_b = (
-            (bgcolor >> 24) & 0xFF,
-            (bgcolor >> 16) & 0xFF,
-            (bgcolor >> 8) & 0xFF,
-            bgcolor & 0xFF,
+        self._size, self.info["loop"], bgcolor, self.n_frames, self.rawmode = (
+            self._decoder.get_info()
         )
-        self.info["background"] = (bg_r, bg_g, bg_b, bg_a)
-        self._n_frames = frame_count
-        self.mode = "RGB" if mode == "RGBX" else mode
-        self.rawmode = mode
-        self.tile = []
+        self.info["background"] = (
+            (bgcolor >> 16) & 0xFF,  # R
+            (bgcolor >> 8) & 0xFF,  # G
+            bgcolor & 0xFF,  # B
+            (bgcolor >> 24) & 0xFF,  # A
+        )
+        self.is_animated = self.n_frames > 1
+        self._mode = "RGB" if self.rawmode == "RGBX" else self.rawmode
 
         # Attempt to read ICC / EXIF / XMP chunks from file
-        icc_profile = self._decoder.get_chunk("ICCP")
-        exif = self._decoder.get_chunk("EXIF")
-        xmp = self._decoder.get_chunk("XMP ")
-        if icc_profile:
-            self.info["icc_profile"] = icc_profile
-        if exif:
-            self.info["exif"] = exif
-        if xmp:
-            self.info["xmp"] = xmp
+        for key, chunk_name in {
+            "icc_profile": "ICCP",
+            "exif": "EXIF",
+            "xmp": "XMP ",
+        }.items():
+            if value := self._decoder.get_chunk(chunk_name):
+                self.info[key] = value
 
         # Initialize seek state
         self._reset(reset=False)
-        self.seek(0)
 
-    def _getexif(self):
+    def _getexif(self) -> dict[int, Any] | None:
         if "exif" not in self.info:
             return None
-        return dict(self.getexif())
+        return self.getexif()._get_merged_dict()
 
-    @property
-    def n_frames(self):
-        return self._n_frames
-
-    @property
-    def is_animated(self):
-        return self._n_frames > 1
-
-    def seek(self, frame):
-        if not _webp.HAVE_WEBPANIM:
-            return super().seek(frame)
-
-        # Perform some simple checks first
-        if frame >= self._n_frames:
-            raise EOFError("attempted to seek beyond end of sequence")
-        if frame < 0:
-            raise EOFError("negative frame index is not valid")
+    def seek(self, frame: int) -> None:
+        if not self._seek_check(frame):
+            return
 
         # Set logical frame to requested position
         self.__logical_frame = frame
 
-    def _reset(self, reset=True):
+    def _reset(self, reset: bool = True) -> None:
         if reset:
             self._decoder.reset()
         self.__physical_frame = 0
         self.__loaded = -1
         self.__timestamp = 0
 
-    def _get_next(self):
+    def _get_next(self) -> tuple[bytes, int, int]:
         # Get next frame
         ret = self._decoder.get_next()
         self.__physical_frame += 1
@@ -132,7 +101,8 @@ class WebPImageFile(ImageFile.ImageFile):
         if ret is None:
             self._reset()  # Reset just to be safe
             self.seek(0)
-            raise EOFError("failed to decode next frame in WebP file")
+            msg = "failed to decode next frame in WebP file"
+            raise EOFError(msg)
 
         # Compute duration
         data, timestamp = ret
@@ -143,7 +113,7 @@ class WebPImageFile(ImageFile.ImageFile):
         timestamp -= duration
         return data, timestamp, duration
 
-    def _seek(self, frame):
+    def _seek(self, frame: int) -> None:
         if self.__physical_frame == frame:
             return  # Nothing to do
         if frame < self.__physical_frame:
@@ -151,33 +121,37 @@ class WebPImageFile(ImageFile.ImageFile):
         while self.__physical_frame < frame:
             self._get_next()  # Advance to the requested frame
 
-    def load(self):
-        if _webp.HAVE_WEBPANIM:
-            if self.__loaded != self.__logical_frame:
-                self._seek(self.__logical_frame)
+    def load(self) -> Image.core.PixelAccess | None:
+        if self.__loaded != self.__logical_frame:
+            self._seek(self.__logical_frame)
 
-                # We need to load the image data for this frame
-                data, timestamp, duration = self._get_next()
-                self.info["timestamp"] = timestamp
-                self.info["duration"] = duration
-                self.__loaded = self.__logical_frame
+            # We need to load the image data for this frame
+            data, self.info["timestamp"], self.info["duration"] = self._get_next()
+            self.__loaded = self.__logical_frame
 
-                # Set tile
-                if self.fp and self._exclusive_fp:
-                    self.fp.close()
-                self.fp = BytesIO(data)
-                self.tile = [("raw", (0, 0) + self.size, 0, self.rawmode)]
+            # Set tile
+            if self.fp and self._exclusive_fp:
+                self.fp.close()
+            self.fp = BytesIO(data)
+            self.tile = [ImageFile._Tile("raw", (0, 0) + self.size, 0, self.rawmode)]
 
         return super().load()
 
-    def tell(self):
-        if not _webp.HAVE_WEBPANIM:
-            return super().tell()
+    def load_seek(self, pos: int) -> None:
+        pass
 
+    def tell(self) -> int:
         return self.__logical_frame
 
 
-def _save_all(im, fp, filename):
+def _convert_frame(im: Image.Image) -> Image.Image:
+    # Make sure image mode is supported
+    if im.mode not in ("RGBX", "RGBA", "RGB"):
+        im = im.convert("RGBA" if im.has_transparency_data else "RGB")
+    return im
+
+
+def _save_all(im: Image.Image, fp: IO[bytes], filename: str | bytes) -> None:
     encoderinfo = im.encoderinfo.copy()
     append_images = list(encoderinfo.get("append_images", []))
 
@@ -190,7 +164,7 @@ def _save_all(im, fp, filename):
         _save(im, fp, filename)
         return
 
-    background = (0, 0, 0, 0)
+    background: int | tuple[int, ...] = (0, 0, 0, 0)
     if "background" in encoderinfo:
         background = encoderinfo["background"]
     elif "background" in im.info:
@@ -201,9 +175,11 @@ def _save_all(im, fp, filename):
             palette = im.getpalette()
             if palette:
                 r, g, b = palette[background * 3 : (background + 1) * 3]
-                background = (r, g, b, 0)
+                background = (r, g, b, 255)
+            else:
+                background = (background, background, background, 255)
 
-    duration = im.encoderinfo.get("duration", 0)
+    duration = im.encoderinfo.get("duration", im.info.get("duration", 0))
     loop = im.encoderinfo.get("loop", 0)
     minimize_size = im.encoderinfo.get("minimize_size", False)
     kmin = im.encoderinfo.get("kmin", None)
@@ -212,8 +188,9 @@ def _save_all(im, fp, filename):
     verbose = False
     lossless = im.encoderinfo.get("lossless", False)
     quality = im.encoderinfo.get("quality", 80)
+    alpha_quality = im.encoderinfo.get("alpha_quality", 100)
     method = im.encoderinfo.get("method", 0)
-    icc_profile = im.encoderinfo.get("icc_profile", "")
+    icc_profile = im.encoderinfo.get("icc_profile") or ""
     exif = im.encoderinfo.get("exif", "")
     if isinstance(exif, Image.Exif):
         exif = exif.tobytes()
@@ -231,12 +208,10 @@ def _save_all(im, fp, filename):
     if (
         not isinstance(background, (list, tuple))
         or len(background) != 4
-        or not all(v >= 0 and v < 256 for v in background)
+        or not all(0 <= v < 256 for v in background)
     ):
-        raise OSError(
-            "Background color is not an RGBA tuple clamped to (0-255): %s"
-            % str(background)
-        )
+        msg = f"Background color is not an RGBA tuple clamped to (0-255): {background}"
+        raise OSError(msg)
 
     # Convert to packed uint
     bg_r, bg_g, bg_b, bg_a = background
@@ -244,8 +219,7 @@ def _save_all(im, fp, filename):
 
     # Setup the WebP animation encoder
     enc = _webp.WebPAnimEncoder(
-        im.size[0],
-        im.size[1],
+        im.size,
         background,
         loop,
         minimize_size,
@@ -261,38 +235,21 @@ def _save_all(im, fp, filename):
     cur_idx = im.tell()
     try:
         for ims in [im] + append_images:
-            # Get # of frames in this image
+            # Get number of frames in this image
             nfr = getattr(ims, "n_frames", 1)
 
             for idx in range(nfr):
                 ims.seek(idx)
-                ims.load()
 
-                # Make sure image mode is supported
-                frame = ims
-                rawmode = ims.mode
-                if ims.mode not in _VALID_WEBP_MODES:
-                    alpha = (
-                        "A" in ims.mode
-                        or "a" in ims.mode
-                        or (ims.mode == "P" and "A" in ims.im.getpalettemode())
-                    )
-                    rawmode = "RGBA" if alpha else "RGB"
-                    frame = ims.convert(rawmode)
-
-                if rawmode == "RGB":
-                    # For faster conversion, use RGBX
-                    rawmode = "RGBX"
+                frame = _convert_frame(ims)
 
                 # Append the frame to the animation encoder
                 enc.add(
-                    frame.tobytes("raw", rawmode),
-                    timestamp,
-                    frame.size[0],
-                    frame.size[1],
-                    rawmode,
+                    frame.getim(),
+                    round(timestamp),
                     lossless,
                     quality,
+                    alpha_quality,
                     method,
                 )
 
@@ -307,46 +264,47 @@ def _save_all(im, fp, filename):
         im.seek(cur_idx)
 
     # Force encoder to flush frames
-    enc.add(None, timestamp, 0, 0, "", lossless, quality, 0)
+    enc.add(None, round(timestamp), lossless, quality, alpha_quality, 0)
 
     # Get the final output from the encoder
     data = enc.assemble(icc_profile, exif, xmp)
     if data is None:
-        raise OSError("cannot write file as WebP (encoder returned None)")
+        msg = "cannot write file as WebP (encoder returned None)"
+        raise OSError(msg)
 
     fp.write(data)
 
 
-def _save(im, fp, filename):
+def _save(im: Image.Image, fp: IO[bytes], filename: str | bytes) -> None:
     lossless = im.encoderinfo.get("lossless", False)
     quality = im.encoderinfo.get("quality", 80)
-    icc_profile = im.encoderinfo.get("icc_profile", "")
-    exif = im.encoderinfo.get("exif", "")
+    alpha_quality = im.encoderinfo.get("alpha_quality", 100)
+    icc_profile = im.encoderinfo.get("icc_profile") or ""
+    exif = im.encoderinfo.get("exif", b"")
     if isinstance(exif, Image.Exif):
         exif = exif.tobytes()
+    if exif.startswith(b"Exif\x00\x00"):
+        exif = exif[6:]
     xmp = im.encoderinfo.get("xmp", "")
+    method = im.encoderinfo.get("method", 4)
+    exact = 1 if im.encoderinfo.get("exact") else 0
 
-    if im.mode not in _VALID_WEBP_LEGACY_MODES:
-        alpha = (
-            "A" in im.mode
-            or "a" in im.mode
-            or (im.mode == "P" and "A" in im.im.getpalettemode())
-        )
-        im = im.convert("RGBA" if alpha else "RGB")
+    im = _convert_frame(im)
 
     data = _webp.WebPEncode(
-        im.tobytes(),
-        im.size[0],
-        im.size[1],
+        im.getim(),
         lossless,
         float(quality),
-        im.mode,
+        float(alpha_quality),
         icc_profile,
+        method,
+        exact,
         exif,
         xmp,
     )
     if data is None:
-        raise OSError("cannot write file as WebP (encoder returned None)")
+        msg = "cannot write file as WebP (encoder returned None)"
+        raise OSError(msg)
 
     fp.write(data)
 
@@ -354,7 +312,6 @@ def _save(im, fp, filename):
 Image.register_open(WebPImageFile.format, WebPImageFile, _accept)
 if SUPPORTED:
     Image.register_save(WebPImageFile.format, _save)
-    if _webp.HAVE_WEBPANIM:
-        Image.register_save_all(WebPImageFile.format, _save_all)
+    Image.register_save_all(WebPImageFile.format, _save_all)
     Image.register_extension(WebPImageFile.format, ".webp")
     Image.register_mime(WebPImageFile.format, "image/webp")
